@@ -15,6 +15,9 @@
  *                                                                         *
  ***************************************************************************/
 
+#define __STDC_FORMAT_MACROS
+#include <inttypes.h>
+
 #include "fs_base.h"
 #include "partimage.h"
 #include "imagefile.h"
@@ -78,7 +81,7 @@ void checkForEscape()
 
   if (g_bSigInt == true)
     {
-      nRes = g_interface->msgBoxYesNo(i18n("Exit partimage ?"), i18n("Do you really want to cancel the current operation ?"));
+      nRes = g_interface->msgBoxYesNo(i18n("Exit partimage?"), i18n("Do you really want to cancel the current operation?"));
       if (nRes == MSGBOX_YES)
 	{
 	  errno = 0;
@@ -131,6 +134,7 @@ void CFSBase::saveUsedBlocksToImageFile(COptions *options)
   check.cMagic[0] = 'C';
   check.cMagic[1] = 'H';
   check.cMagic[2] = 'K';
+  check.cMagic[3] = 0;
 
   dwStats = 0;
   i = 0;
@@ -345,7 +349,7 @@ void CFSBase::restoreUsedBlocksToDisk(COptions *options)
 		      showDebug(1, "CHECK-MAGIC error %3s %3s\n", "CHK", check.cMagic);
 		      nRes=g_interface->msgBoxYesNo(i18n("local check error"),
                          i18n("Impossible to find a local check structure. "
-                         "Data may be damaged. Do you want to continue ?"));
+                         "Data may be damaged. Do you want to continue?"));
 		      if (nRes == MSGBOX_NO)
 		        THROW(ERR_WRONGMAGIC);
 		    }
@@ -358,7 +362,7 @@ void CFSBase::restoreUsedBlocksToDisk(COptions *options)
 		     
 		          nRes=g_interface->msgBoxYesNo(i18n("local CRC error"),
                              i18n("There was a position error in CCheck. "
-                             "Data may be damaged. Do you want to continue ?"));
+                             "Data may be damaged. Do you want to continue?"));
 		          if (nRes == MSGBOX_NO)
 			    THROW(ERR_CHECK_NUM, i, check.qwPos);
 		        }
@@ -370,7 +374,7 @@ void CFSBase::restoreUsedBlocksToDisk(COptions *options)
 
 		          nRes=g_interface->msgBoxYesNo(i18n("local CRC error"),
                              i18n("There was a CRC error in CCheck. "
-                             "Data may be damaged. Do you want to continue ?"));
+                             "Data may be damaged. Do you want to continue?"));
 		          if (nRes == MSGBOX_NO)
 			    THROW(ERR_CHECK_CRC, dwCRC, check.dwCRC);
 		        }
@@ -563,7 +567,7 @@ void CFSBase::saveToImage(CImage *image, COptions *options, CSavingWindow *gui)
   showDebug(3, "TRACE: 007 (copyUsedBlocks)\n");  
   g_interface -> StatusLine(i18n("copying used data blocks"));
   if (strlen(options->szFullyBatchMode)>0)
-      fprintf(stderr,"File Name    Size     T:Elapsed/Estimated  Rate/min     Progress\n");
+      fprintf(stderr,i18n("File Name    Size     T:Elapsed/Estimated  Rate/min     Progress\n"));
 
   showDebug(3, "begin of saveUsedBlocksToImageFile()\n");  
   try { saveUsedBlocksToImageFile(options); }
@@ -701,7 +705,7 @@ void CFSBase::restoreFromImage(CImage *image, COptions *options, CRestoringWindo
   errno = 0;
   g_interface -> StatusLine(i18n("copying used data blocks"));
   if(strlen(options->szFullyBatchMode)>0)
-     fprintf(stderr,"File name    Level    T:Elapsed/Estimated  Rate/min     Progress\n");
+     fprintf(stderr,i18n("File name    Level    T:Elapsed/Estimated  Rate/min     Progress\n"));
   showDebug(1, "begin of restoreUsedBlocksToDisk()\n");  
   try { restoreUsedBlocksToDisk(options); }
   catch ( CExceptions * excep )
@@ -726,10 +730,10 @@ int CFSBase::getStdInfos(char *szDest, int nMaxLen, bool bShowBlocksInfo)
 
   if (bShowBlocksInfo)
     {
-      SNPRINTF(szBlocksInfo, i18n("Block size....................%llu bytes\n"
-				  "Total blocks count............%llu\n"
-				  "Used blocks count.............%llu\n"
-				  "Free blocks count.............%llu\n"),
+      SNPRINTF(szBlocksInfo, i18n("Block size:...................%"PRIu64" bytes\n"
+				  "Total blocks count:...........%"PRIu64"\n"
+				  "Used blocks count:............%"PRIu64"\n"
+				  "Free blocks count:............%"PRIu64"\n"),
 	       m_header.qwBlockSize, 
 	       m_header.qwBlocksCount,
 	       m_header.qwUsedBlocks,
@@ -741,11 +745,11 @@ int CFSBase::getStdInfos(char *szDest, int nMaxLen, bool bShowBlocksInfo)
     }
  
   snprintf(szDest, nMaxLen, i18n("%s" // Blocks infos
-				 "Space usage:..................%llu %%\n"
-				 "Used space....................%s\n"
-				 "Free space....................%s\n"
-				 "Bitmap size...................%s\n"
-				 "Label.........................%s\n"),
+				 "Space usage:..................%"PRIu64" %%\n"
+				 "Used space:...................%s\n"
+				 "Free space:...................%s\n"
+				 "Bitmap size:..................%s\n"
+				 "Label:........................%s\n"),
 	   szBlocksInfo,
 	   (100 * m_header.qwUsedBlocks)/m_header.qwBlocksCount, 
 	   formatSize(m_header.qwUsedBlocks * m_header.qwBlockSize, szTemp1),
@@ -769,6 +773,7 @@ int CFSBase::calculateSpaceFromBitmap()
   QWORD qwFreeBlocksCount = 0;
   QWORD i;
 
+  showDebug(1, "BITMAP: m_header.qwBlocksCount = %llu\n", m_header.qwBlocksCount);
   for (i=0; i < m_header.qwBlocksCount; i++)
     {
       if (m_bitmap.isBitSet(i))
