@@ -61,31 +61,38 @@ CImage::CImage(COptions * options)
   else
     {
 #ifdef MUST_LOGIN
-      if (strlen(options -> szUserName) && strlen(options -> szPassWord))
+      if (options->bMustLogin)
         {
-          strncpy(login, options->szUserName, 2048);
-          strncpy(passwd, options->szPassWord, 2048);
-          *(login+2047) = '\0';
-          *(passwd+2047) = '\0';
-        }
-      else 
-        {
-// we get login/pass from user
-          int nRes;
-          nRes = g_interface -> askLogin(login, passwd, 2048);
-          if (nRes)
+          if (strlen(options -> szUserName) && strlen(options -> szPassWord))
             {
-              free(login);
-              free(passwd);
-              THROW( ERR_CANCELED);
+              strncpy(login, options->szUserName, 2048);
+              strncpy(passwd, options->szPassWord, 2048);
+              *(login+2047) = '\0';
+              *(passwd+2047) = '\0';
             }
-        }	
-#else
+          else 
+            {
+    // we get login/pass from user
+              int nRes;
+              nRes = g_interface -> askLogin(login, passwd, 2048);
+              if (nRes)
+                {
+                  free(login);
+                  free(passwd);
+                  THROW( ERR_CANCELED);
+                }
+            }	
+        }
+      else
+        {
+#endif
 // fake login/passwd
       strcpy(login, "nologin");
       strcpy(passwd, "nopass");
+#ifdef MUST_LOGIN
+        }
 #endif
-		
+
       m_cid = new CImageNet(options, options->bUseSSL);
       try { m_cid -> Connect(options, login, passwd, options->szServerName,
          options->dwServerPort); }
@@ -186,23 +193,23 @@ void CImage::set_bIsOpened(bool n)
 {
   m_cid -> set_bIsOpened(n);
 }
-void CImage::set_szPath(char * str)
+void CImage::set_szPath(const char * str)
 {
   m_cid -> set_szPath(str);
 }
-void CImage::set_szImageFilename(char * str)
+void CImage::set_szImageFilename(const char * str)
 {
   m_cid -> set_szImageFilename(str);
 }
-void CImage::set_szOriginalFilename(char * str)
+void CImage::set_szOriginalFilename(const char * str)
 {
   m_cid -> set_szOriginalFilename(str);
 }
-void CImage::set_szImageFilename(char * str, int n)
+void CImage::set_szImageFilename(const char * str, int n)
 {
   m_cid -> set_szImageFilename(str, n);
 }
-void CImage::set_szOriginalFilename(char * str, int n)
+void CImage::set_szOriginalFilename(const char * str, int n)
 {
   m_cid -> set_szOriginalFilename(str, n);
 }
@@ -270,7 +277,7 @@ void CImage::writeSplit()
     { qwFreeSpace = 0LL; }*/
       
   m_guiSave -> showImageFileInfo(get_szImageFilename(), qwFreeSpace, 0, m_options.szFullyBatchMode);
-  showDebug(2, "imagefile split done\n");
+  showDebug(2, "image file split done\n");
 
   RETURN;
 }
@@ -317,7 +324,7 @@ void CImage::write(void *vBuf, DWORD dwLength, bool bUpdateCRC)
       
       m_guiSave -> showImageFileInfo(get_szImageFilename(), qwFreeSpace, 0, m_options.szFullyBatchMode);
       qwTotalBytes = 0LL;
-      showDebug(2, "imagefile split done\n");
+      showDebug(2, "image file split done\n");
     }
   
   // ---- fatal error
@@ -361,7 +368,7 @@ void CImage::write(void *vBuf, DWORD dwLength, bool bUpdateCRC)
     }
 
   qwCount += (QWORD) dwRes;
-  if (qwCount > (1024 * 1024)) // update the window every 1 MB
+  if (qwCount > (1024 * 1024)) // update the window every 1 MiB
     qwCount = 0LL;
 
   showDebug(3, "write end -> %ld\n", dwLength);
@@ -438,7 +445,7 @@ void CImage::read(char *cBuf, DWORD dwLength, bool bUpdateCRC)
 
   // ---- update the window ----
   qwCount += (QWORD) dwLength;
-  if (qwCount > (1024 * 1024)) // update the window every 1 MB
+  if (qwCount > (1024 * 1024)) // update the window every 1 MiB
     {
       qwCount = 0LL;
       if (m_guiRestore)
@@ -475,7 +482,7 @@ void CImage::splitChangeImageFileWrite()
       if (!fork()) // we're the child // no error check maid
         execl("/tmp/partimage-shell", 
            get_szImageFilename(), get_szPath(),
-           get_szOriginalFilename(), get_dwVolumeNumber());
+           get_szOriginalFilename(), get_dwVolumeNumber(),(char*)NULL);
     }
 
   // wait for a keyboard keypress if need
@@ -488,7 +495,7 @@ void CImage::splitChangeImageFileWrite()
     }
 
   set_dwVolumeNumber(get_dwVolumeNumber()+1); // increase the volume number
-  SNPRINTF(aux, "%s/%s.%.3ld", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
+  SNPRINTF(aux, "%s/%s.%.3d", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
   //*(aux+MAXPATHLEN-1) = '\0';
   
   set_szImageFilename(aux);
@@ -514,7 +521,7 @@ void CImage::splitChangeImageFileRead()
 
   // increase volume number
   set_dwVolumeNumber(get_dwVolumeNumber()+1);
-  SNPRINTF(aux, "%s/%s.%.3ld", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
+  SNPRINTF(aux, "%s/%s.%.3d", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
   //*(aux+MAXPATHLEN-1) = '\0';
   set_szImageFilename(aux);
 
@@ -562,7 +569,7 @@ void CImage::closeReading(bool bForceExit /*  = false */)
   showDebug(3, "TRACE_001\n");
 
   //pthread_join(g_threadBuffer, NULL);
-  // MULTI-THREAD TERMISAISON
+  // MULTI-THREAD TERMINAISON
   if (m_options.dwCompression == COMPRESS_NONE) // No compression
     nRes = fclose(m_fImageFile);	
   else if (m_options.dwCompression == COMPRESS_GZIP) // Gzip compression
@@ -700,7 +707,7 @@ int CImage::openWritingFdDisk()
 	      set_szPath(aux2);
 	      extractFilenameFromFullPath(szAux, aux2); // filename without path
 	      set_szOriginalFilename(aux2);
-	      SNPRINTF(szAux, "%s/%s.%.3ld", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
+	      SNPRINTF(szAux, "%s/%s.%.3d", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
 	      set_szImageFilename(szAux);
 	      nRes = openWritingFdDisk();
 	      RETURN_int(nRes);
@@ -728,7 +735,7 @@ int CImage::openWritingFdDisk()
             case ERR_RETRY:
               strcpy(szAux, excep->getNewString()); // endding '/' were removed 
               set_szPath(szAux); 
-	      SNPRINTF(aux2, "%s/%s.%.3ld", szAux, get_szOriginalFilename(),
+	      SNPRINTF(aux2, "%s/%s.%.3d", szAux, get_szOriginalFilename(),
 		       get_dwVolumeNumber());
               set_szImageFilename(aux2);
 	      
@@ -941,9 +948,9 @@ void CImage::readAndCheckCRC(char *cData, DWORD dwDataLen)
 
 // =======================================================
 // may throw incomplete, errno, opened, exist, access, full, canceled
-void CImage::writeMagic(char *szMagicString)
+void CImage::writeMagic(const char *szMagicString)
 {
-  try { write(szMagicString, strlen(szMagicString), true); }
+  try { write((void *) szMagicString, strlen(szMagicString), true); }
   catch (CExceptions * execp)
     { throw execp; }
 }
@@ -952,7 +959,7 @@ void CImage::writeMagic(char *szMagicString)
 // may throw wrongmagic
 // may throw opened, eof, errno, canceled, volumeheader, volumenumber,
 //           partitionvolume
-void CImage::readAndCheckMagic(char *szMagicString)
+void CImage::readAndCheckMagic(const char *szMagicString)
 {
   char szAux [strlen(szMagicString)+1];
 
@@ -1068,7 +1075,7 @@ void CImage::openReading(CVolumeHeader *vh /* = NULL */)
             set_szPath(aux2);
             extractFilenameFromFullPath(szAux, aux2);
             set_szOriginalFilename(aux2);
-            SNPRINTF(szAux, "%s/%s.%.3ld", get_szPath(), 
+            SNPRINTF(szAux, "%s/%s.%.3d", get_szPath(), 
                get_szOriginalFilename(), get_dwVolumeNumber());
             set_szImageFilename(szAux);
             umountImageLocation();
@@ -1132,12 +1139,12 @@ void CImage::openReading(CVolumeHeader *vh /* = NULL */)
 	
   if (strcmp(headVolume.szMagicString, MAGIC_BEGIN_VOLUME) != 0)
     {	
-      SNPRINTF(szAux, "%s/%s.%.3ld", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
+      SNPRINTF(szAux, "%s/%s.%.3d", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
       THROW(ERR_NOTAPARTIMAGEFILE, szAux);
     }
 
   // ----------- check image version
-  showDebug(4, "ImageFile version: [%s] and CURRENT_IMAGE_FORMAT=[%s]\n", headVolume.szVersion, CURRENT_IMAGE_FORMAT);
+  showDebug(4, "image file version: [%s] and CURRENT_IMAGE_FORMAT=[%s]\n", headVolume.szVersion, CURRENT_IMAGE_FORMAT);
   if (strcmp(headVolume.szVersion, CURRENT_IMAGE_FORMAT) > 0)
     THROW(ERR_VERSION_NEWER, headVolume.szVersion);
   else if ((strcmp(headVolume.szVersion, CURRENT_IMAGE_FORMAT) < 0) && 
