@@ -29,6 +29,10 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
+
+#define __STDC_FORMAT_MACROS
+#include <inttypes.h>
+
 #include <sched.h>
 #include <sys/types.h>
 #include <sys/param.h>
@@ -86,6 +90,7 @@ static struct option const long_options[] =
   {"compilinfo", no_argument, NULL, 'i'},
   {"nombr", no_argument, NULL, 'M'},
   {"allowmnt", no_argument, NULL, 'm'},
+  {"nologin", no_argument, NULL, 'L'},
   {"nossl", no_argument, NULL, 'n'},
   {"overwrite", no_argument, NULL, 'o'},
   {"password", required_argument, NULL, 'P'},
@@ -105,7 +110,7 @@ static struct option const long_options[] =
 
 static char finish_last_countfile_lock[]="/tmp/partimageXXXXXX.lock";
 static char finish_last_countfile[]="/tmp/partimageXXXXXX";
-static char optstring[]="z:oV:ecmdhf:s:p:bwg:vynSMa:iU:P:XB:";
+static char optstring[]="z:oV:ecmdhf:s:p:bwg:vyLnSMa:iU:P:XB:";
 FILE * g_fDebug; // debug file
 FILE * g_fLocalDebug; // debug file
 CInterface * g_interface;
@@ -135,11 +140,18 @@ static void catch_sigint(int signo)
     }*/
 }
 
+void ensure_interface_is_non_interactive()
+{
+  delete g_interface;
+  g_interface = new CInterfaceNone((char *)"");
+}
+
 // =======================================================
 int main(int argc, char *argv[])
 {
   int nRes;
   int nOptCh;
+  int iRet __attribute__ ((unused)) ;
   COptions options;
   int nChoice;
   char szDevice[MAX_DEVICENAMELEN];
@@ -161,6 +173,7 @@ int main(int argc, char *argv[])
   // initialize options with defaults values
   memset(&options, 0, sizeof(COptions));
   options.bUseSSL = OPTIONS_DEFAULT_SSL;
+  options.bMustLogin = OPTIONS_DEFAULT_LOGIN;
   options.bBackupMBR = OPTIONS_DEFAULT_BACKUP_MBR;
   options.bSimulateMode = OPTIONS_DEFAULT_SIMULATE_MODE;
   options.bOverwrite = OPTIONS_DEFAULT_OVERWRITE;
@@ -205,37 +218,46 @@ int main(int argc, char *argv[])
 	    strncpy(options.szServerName, optarg, MAX_HOSTNAMESIZE);
             *(options.szServerName+MAX_HOSTNAMESIZE-1) = '\0';
 	    break;
+
 	  case 'a': // auto mount options
 	    strncpy(options.szAutoMount, optarg, MAXPATHLEN);
             *(options.szAutoMount+MAXPATHLEN-1) = '\0';
 	    break;
+
 	  case 'U': // username for serverauth
             strncpy(options.szUserName, optarg, MAX_USERNAMELEN);
             *(options.szUserName+MAX_USERNAMELEN-1) = '\0';
             break;
+
 	  case 'B': // fully batchmode: don't do any newt or anything
 	    strncpy(options.szFullyBatchMode, optarg, 2047);
 	    options.bBatchMode = true;
 	    break;
+
 	  case 'P': // password for serverauth
             strncpy(options.szPassWord, optarg, MAX_PASSWORDLEN);
             *(options.szPassWord+MAX_PASSWORDLEN-1) = '\0';
             break;
+
 	  case 'b': // batchmode: don't wait for user action (return)
 	    options.bBatchMode = true;
 	    break;
+
 	  case 'w': // wait after each new colume
 	    options.bSplitWait = true;
 	    break;
           case 'X': // run shell after all volume
             options.bRunShell = true;
             break; 
+
 	  case 'M': // do not create a backup of the MBR
 	    options.bBackupMBR = false;
 	    break;
+
 	  case 'y': // no sync at the end
 	    options.bSync = false;
 	    break;
+
 	  case 'p': // partimaged server's listening port
 	    options.dwServerPort = atol(optarg);
 	    if ((options.dwServerPort < 1) || (options.dwServerPort > 65535))
@@ -245,6 +267,7 @@ int main(int argc, char *argv[])
 		return EXIT_FAILURE;
 	      }
 	    break;
+
 	  case 'z': // compression level
 	    options.dwCompression = atol(optarg);
 	    if ((options.dwCompression < 0) || (options.dwCompression > 2))
@@ -278,7 +301,7 @@ int main(int argc, char *argv[])
 	  case 'V': // split image into multiple files
 	  
 	    options.qwSplitSize = (QWORD)atol(optarg) * 1024 * 1024;
-	    fprintf (stderr, i18n("Volume size: %llu bytes (%ld MiB)\n"),
+	    fprintf (stderr, i18n("Volume size: %"PRIu64" bytes (%ld MiB)\n"),
 		     options.qwSplitSize, atol(optarg));
 	    break;
 	  
@@ -319,9 +342,14 @@ int main(int argc, char *argv[])
 	    return EXIT_SUCCESS;
 	    break;
 
+	  case 'L': // no login
+	    options.bMustLogin = false;
+	    break;
+
 	  case 'n': // no ssl
 	    options.bUseSSL = false;
 	    break;
+
 	  case 'S': // simulate
 	    options.bSimulateMode = true;
 	    break;
@@ -339,7 +367,7 @@ int main(int argc, char *argv[])
       SNPRINTF(szAux, "%s", PARTIMAGE_LOG);
 #endif 
       unlink(szAux2);
-      symlink(szAux, szAux2);
+      iRet = symlink(szAux, szAux2);
       g_fDebug = openFileDescriptorSecure(szAux, "a", O_WRONLY | O_CREAT | O_NOFOLLOW | O_TRUNC, S_IRUSR | S_IWUSR);
 
       g_nDebugThreadMain = getpid();
@@ -517,6 +545,10 @@ int main(int argc, char *argv[])
 	nChoice = OPERATION_SAVE_ALL;
       else if (strcmp(argv[optind], "rest_all")==0)
 	nChoice = OPERATION_REST_ALL;
+      else if (strcmp(argv[optind], "save_file")==0)
+	nChoice = OPERATION_SAVE_FILE;
+      else if (strcmp(argv[optind], "rest_file")==0)
+	nChoice = OPERATION_REST_FILE;
 
       strncpy(nChoice == OPERATION_REST_ALL || nChoice == OPERATION_SAVE_ALL ? szImageDir : szImageFile, argv[optind+1], MAXPATHLEN);      
     }
@@ -535,7 +567,7 @@ int main(int argc, char *argv[])
     g_interface->msgBoxCancel(i18n("Warning"),
        i18n("You have devfs enabled but not mounted. You have to do it to "
             "continue. You can also boot with non-devfs kernel if you have "
-	    "one."));
+            "one."));
     delete g_interface;
     return EXIT_FAILURE;
   }
@@ -575,8 +607,9 @@ int main(int argc, char *argv[])
 		  goto beginMainWin;
 		}
 
-	      if (!options.bBatchMode)
-		g_interface -> Error(excep, szImageFile, szDevice);
+	      if (options.bBatchMode) ensure_interface_is_non_interactive();
+
+              g_interface -> Error(excep, szImageFile, szDevice);
 
 	      showDebug(1, "\nFINAL ERROR\n\n");
 	  
@@ -587,8 +620,11 @@ int main(int argc, char *argv[])
       else
 	{
 	  showDebug(1, "The file system of [%s] is [%s], and is not supported\n", szDevice, szFileSystem);
-	  if (!options.bBatchMode)
-	    g_interface->msgBoxError("The file system of [%s] is [%s], and is not supported", szDevice, szFileSystem);
+
+          if (options.bBatchMode) ensure_interface_is_non_interactive();
+
+          g_interface->msgBoxError(i18n("The file system of [%s] is [%s], and is not supported"), szDevice, szFileSystem);
+
 	  nRes = -1;
 	}
 //#endif // DEVEL_SUPPORT
@@ -635,7 +671,35 @@ int main(int argc, char *argv[])
 	  nRes = -1;
 	}
       break;
-      
+
+    case OPERATION_SAVE_FILE:
+      showDebug(1, "action=SAVE_FILE\n");
+      try { save_file(szImageFile, &options); }
+      catch (CExceptions *excep)
+	{
+          showDebug(1, "save_file caught exception: %d\n", excep->GetExcept());
+
+	  if (options.bBatchMode) ensure_interface_is_non_interactive();	      
+	  g_interface -> Error(excep, szImageFile, "");
+
+	  nRes = -1;
+	}
+      break;
+
+    case OPERATION_REST_FILE:
+      showDebug(1, "action=REST_FILE\n");
+      try { rest_file(szImageFile, &options); }
+      catch (CExceptions *excep)
+	{
+          showDebug(1, "rest_file caught exception: %d\n", excep->GetExcept());
+
+	  if (options.bBatchMode) ensure_interface_is_non_interactive();	      
+	  g_interface -> Error(excep, szImageFile, "");
+
+	  nRes = -1;
+	}
+      break;
+
     case OPERATION_IMGINFO:
       showDebug(1, "action=IMGINFO\n");
       try { showImgInfos(szImageFile, &options); }
@@ -692,7 +756,7 @@ int main(int argc, char *argv[])
         {
           if (options.bBatchMode)
                fprintf(stderr,"\n\n");
-          g_interface -> StatusLine(i18n("commiting buffer cache to disk."));
+          if (!options.bBatchMode) g_interface -> StatusLine(i18n("commiting buffer cache to disk."));
 	  sync();
         }
       closeDebugFiles();
@@ -823,15 +887,16 @@ int main(int argc, char *argv[])
 	  {
 	    if ((fd_lock = mkstemp(finish_last_countfile_lock)) == -1)
 	      {
-		delete g_interface;
-		fprintf(stderr, i18n("Error: Cannot create lock for countfile"));
-		return EXIT_FAILURE;
-	      }
+			--nRetries;
+			sleep(2);
+			continue;
+		  }
 	    fLockFile = fdopen(fd_lock, "r");
 	    if (fLockFile == NULL)
  	      {
-                 --nRetries;
- 		sleep(2);
+			close(fd_lock);
+			unlink(finish_last_countfile_lock);
+			sleep(2);
  	      }
 	  } while (fLockFile == NULL && nRetries);
  	  if (!nRetries)
@@ -842,6 +907,10 @@ int main(int argc, char *argv[])
  	    }
 	  if ((fd_countfile = mkstemp(finish_last_countfile)) == -1)
 	    {
+			fclose(fLockFile);
+			close(fd_lock);
+			unlink(finish_last_countfile_lock);
+
 	      delete g_interface;
 	      fprintf(stderr, i18n("Error: Cannot create countfile"));
 	      return EXIT_FAILURE;
@@ -849,11 +918,17 @@ int main(int argc, char *argv[])
           fCountFile = fdopen(fd_countfile, "r");
 	  if (fCountFile == NULL)
 	    {
+			fclose(fLockFile);
+			close(fd_lock);
+			close(fd_countfile);
+			unlink(finish_last_countfile_lock);
+			unlink(finish_last_countfile);
+
     	      delete g_interface; 
 	      fprintf(stderr, i18n("Error: Cannot count remaining partimages"));
 	      return EXIT_SUCCESS;
 	    }
-	  fscanf(fCountFile, "%d\n", &nValue);
+	  iRet = fscanf(fCountFile, "%d\n", &nValue);
 	  showDebug(1, "RESTANT: %d\n", nValue);
 	  if (nValue <= 1)
  	    { // we are the last running partimage -> shutdown
@@ -910,7 +985,8 @@ void usage()
   printf("===============================================================================\n");
   printf(i18n("Partition Image (http://www.partimage.org/) version %s [%s]\n"
 	      "---- distributed under the GPL 2 license (GNU General Public License) ----\n\n"
-	      "Supported file systems:....ext2fs, reiserfs, fat16, fat32, ntfs(exp), hpfs\n\n"
+	      "Supported file systems: Ext2/3, ReiserFS, FAT16/32, HPFS, JFS, XFS,\n"
+	      "                        UFS(beta), HFS(beta), NTFS(experimental)\n\n"
 	      "usage: partimage [options] <action> <device> <image_file>\n"
 	      "       partimage <imginfo/restmbr> <image_file>\n\n"
 	      "ex: partimage -z1 -o -d save /dev/hda12 /mnt/backup/redhat-6.2.partimg.gz\n"
@@ -933,31 +1009,32 @@ void usage()
 	      "* -z,  --compress      (image file compression level):\n"
 	      "  -z0, --compress=0    don't compress: very fast but very big image file\n"
 	      "  -z1, --compress=1    compress using gzip: fast and small image file (default)\n"
-	      "  -z2, --compress=2    (compress using bzip2: very slow and very small image file):\n"
+	      "  -z2, --compress=2    compress using bzip2: very slow and very small image file:\n"
 	      "* -c,  --nocheck       don't check the partition before saving\n"
 	      "* -o,  --overwrite     overwrite the existing image file without confirmation\n"
 	      "* -d,  --nodesc        don't ask any description for the image file\n"
 	      "* -V,  --volume        (split image into multiple volumes files)\n"
-	      "  -VX, --volume=X      create volumes with a size of X MB\n"
+	      "  -VX, --volume=X      create volumes with a size of X MiB\n"
 	      "* -w,  --waitvol       wait for a confirmation after each volume change\n"
 	      "* -e,  --erase         erase empty blocks on restore with zero bytes\n"
-	      "* -m,  --allowmnt      don't fail if the partition is mounted. Dangerous !\n"
+	      "* -m,  --allowmnt      don't fail if the partition is mounted. Dangerous!\n"
 	      "* -M,  --nombr         don't create a backup of the MBR (Mast Boot Record) in the image file\n"
 	      "* -h,  --help          show help\n"
 	      "* -v,  --version       show version\n"
 	      "* -i,  --compilinfo    show compilation options used\n"
 	      "* -f,  --finish        (action to do if finished successfully):\n"
 	      "  -f0, --finish=0      wait: don't make anything\n"
-	      "  -f1, --finish=1      halt (power off) the computer\n"
-	      "  -f2, --finish=2      reboot (restart the computer):\n"
+	      "  -f1, --finish=1      halt: power off the computer\n"
+	      "  -f2, --finish=2      reboot: restart the computer\n"
 	      "  -f3, --finish=3      quit\n"
 	      "* -b,  --batch         batch mode: the GUI won't wait for an user action\n"
-              "* -BX, --fully-batch=X batch mode without GUI, X is a challenge response string\n"
-	      "* -y,  --nosync        don't synchronize the disks at the end of the operation (dangerous)\n"
+          "* -BX, --fully-batch=X batch mode without GUI, X is a challenge response string\n"
+	      "* -y,  --nosync        don't synchronize the disks at the end of the operation. Dangerous!\n"
 	      "* -sX, --server=X      give partimaged server's ip address\n"
 	      "* -pX, --port=X        give partimaged server's listening port\n"
-	      "* -g,  --debug=X       set the debug level to X (default: 1):\n"
-	      "* -n,  --nossl         disable SSL in network mode\n"
+	      "* -g,  --debug=X       set the debug level to X (default: 1)\n"
+	      "* -L,  --nologin       disable login in network mode\n"
+ 	      "* -n,  --nossl         disable SSL in network mode\n"
 	      "* -S,  --simulate      simulation of restoration mode\n"
 	      "* -aX, --automnt=X     automatic mount with X options. Read the doc for more details\n"
 	      "* -UX  --username=X    username to authenticate to server\n"
@@ -971,15 +1048,15 @@ int checkStructSizes()
 {
    // ---- check types sizes
 
-   if (sizeof(unsigned long int) != 4)
+   if (sizeof(DWORD) != 4)
    {
-      fprintf (stderr, "Error: sizeof(DWORD) != 4 (%d)\n", sizeof(unsigned long int));
+      fprintf (stderr, i18n("Error: sizeof(DWORD) != 4 (%zd)\n"), sizeof(DWORD));
       goto errcheck;
    }
 
-   if (sizeof(unsigned long long int) != 8)
+   if (sizeof(QWORD) != 8)
    {
-      fprintf (stderr, "Error: sizeof(QWORD) != 8 (%d)\n", sizeof(unsigned long long int));
+      fprintf (stderr, i18n("Error: sizeof(QWORD) != 8 (%zd)\n"), sizeof(QWORD));
       goto errcheck;
    }
 
@@ -988,7 +1065,7 @@ int checkStructSizes()
    // check main header size is 16384
    if (sizeof(CMainHeader) != 16384)
    {
-      fprintf (stderr, "Error: main header size != 16384 (%d)\n",
+      fprintf (stderr, i18n("Error: main header size != 16384 (%zd)\n"),
 	       sizeof(CMainHeader));
       goto errcheck;
    }
@@ -996,7 +1073,7 @@ int checkStructSizes()
    // check ext2 header size is 16384
    if (sizeof(CInfoExt2Header) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: ext2 header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, i18n("Error: ext2 header size != %d (%zd)\n"), INFOS_STRUCT_SIZE,
 	       sizeof(CInfoExt2Header));
       goto errcheck;
    }
@@ -1004,7 +1081,7 @@ int checkStructSizes()
    // check reiserfs header size is 16384
    if (sizeof(CInfoReiserHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: reiserfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, i18n("Error: reiserfs header size != %d (%zd)\n"), INFOS_STRUCT_SIZE,
 	       sizeof(CInfoReiserHeader));
       goto errcheck;
    }
@@ -1012,7 +1089,7 @@ int checkStructSizes()
    // check fat header size is 16384
    if (sizeof(CInfoFatHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: fat header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, i18n("Error: fat header size != %d (%zd)\n"), INFOS_STRUCT_SIZE,
 	       sizeof(CInfoFatHeader));
       goto errcheck;
    }
@@ -1020,7 +1097,7 @@ int checkStructSizes()
    // check ntfs header size is 16384
    if (sizeof(CInfoNtfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: ntfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, i18n("Error: ntfs header size != %d (%zd)\n"), INFOS_STRUCT_SIZE,
 	       sizeof(CInfoNtfsHeader));
       goto errcheck;
    }
@@ -1028,7 +1105,7 @@ int checkStructSizes()
    // check hpfs header size is 16384
    if (sizeof(CInfoHpfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: hpfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, i18n("Error: hpfs header size != %d (%zd)\n"), INFOS_STRUCT_SIZE,
 	       sizeof(CInfoHpfsHeader));
       goto errcheck;
    }
@@ -1036,7 +1113,7 @@ int checkStructSizes()
    // check jfs header size is 16384
    if (sizeof(CInfoJfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: jfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, i18n("Error: jfs header size != %d (%zd)\n"), INFOS_STRUCT_SIZE,
 	       sizeof(CInfoJfsHeader));
       goto errcheck;
    }
@@ -1044,7 +1121,7 @@ int checkStructSizes()
    // check xfs header size is 16384
    if (sizeof(CInfoXfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: xfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, i18n("Error: xfs header size != %d (%zd)\n"), INFOS_STRUCT_SIZE,
 	       sizeof(CInfoXfsHeader));
       goto errcheck;
    }
@@ -1052,7 +1129,7 @@ int checkStructSizes()
    // check hfs header size is 16384
    if (sizeof(CInfoHfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: hfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, i18n("Error: hfs header size != %d (%zd)\n"), INFOS_STRUCT_SIZE,
 	       sizeof(CInfoHfsHeader));
       goto errcheck;
    }
@@ -1060,43 +1137,46 @@ int checkStructSizes()
    // check ufs header size is 16384
    if (sizeof(CInfoUfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: jfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-	       sizeof(CInfoUfsHeader));
+      fprintf (stderr, i18n("Error: jfs header size != %d (%zd)\n"),
+     	 INFOS_STRUCT_SIZE, sizeof(CInfoUfsHeader));
       goto errcheck;
    }
  
    // check tail size is 16384
    if (sizeof(CMainTail) != 16384)
    {	
-      fprintf (stderr, "Error: main footer size != 16384 (%d)\n", sizeof(CMainTail));
+      fprintf (stderr, i18n("Error: main footer size != 16384 (%zd)\n"),
+		sizeof(CMainTail));
       goto errcheck;
    }
   
    // check volume header size is 512
    if (sizeof(CVolumeHeader) != 512)
    {	
-      fprintf (stderr, "Error: volume hedaer size != 512 (%d)\n", sizeof(CVolumeHeader));
+      fprintf (stderr, i18n("Error: volume header size != 512 (%zd)\n"),
+     	 sizeof(CVolumeHeader));
       goto errcheck;
    }
  
    // check MBR size is 2048
    if (sizeof(CMbr) != 2048)
    {	
-      fprintf (stderr, "Error: MBR size != 2048 (%d)\n", sizeof(CMbr));
+      fprintf (stderr, i18n("Error: MBR size != 2048 (%zd)\n"), sizeof(CMbr));
       goto errcheck;
    }
  
    // check volume header size is 512
    if (sizeof(CLocalHeader) != 16384)
    {	
-      fprintf (stderr, "Error: local header size != 16384 (%d)\n", sizeof(CLocalHeader));
+      fprintf (stderr, i18n("Error: local header size != 16384 (%zd)\n"),
+     	 sizeof(CLocalHeader));
       goto errcheck;
    }
 
    return 0; // success
 
   errcheck:
-   printf ("This version has been compiled with an uncompatible version of gcc.\n");
+   printf (i18n("This version has been compiled with an uncompatible version of gcc.\n"));
    //printf("DetailSize: sizeof(DWORD)=%d\n", sizeof(DWORD));
    //printf("DetailSize: sizeof(QWORD)=%d\n", sizeof(QWORD));
    //printf("DetailSize: sizeof(struct tm)=%d\n", sizeof(struct tm));
@@ -1118,7 +1198,7 @@ void formatOperationSucessMsg(char *szMsg, int nMaxLen, char *szOperation)
   fMinElapsed = ((float)timeElapsed) / 60.0;
   qwBytesPerMin = (QWORD) (((float)g_qwCopiedBytesCount) / fMinElapsed);
   
-  snprintf(szMsg, nMaxLen, i18n("%s  successfully finished:\n\n"
+  snprintf(szMsg, nMaxLen, i18n("%s successfully finished:\n\n"
                                 "Time elapsed: %s\n"
                                 "Speed: %s/min\n"
                                 "Data copied: %s"),
