@@ -2,7 +2,7 @@
                           misc.cpp  -  description
                              -------------------
     begin                : Thu Jun 29 2000
-    copyright            : (C) 2000 by Franois Dupoux
+    copyright            : (C) 2000 by François Dupoux
     email                : fdupoux@partimage.org
  ***************************************************************************/
 // $Revision: 1.2 $
@@ -22,6 +22,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#define __STDC_FORMAT_MACROS
+#include <inttypes.h>
 
 #ifdef HAVE_SYS_PARAM_H
   #include <sys/param.h>
@@ -88,8 +90,8 @@ void restoreMbr(char *szImageFile, COptions *options)
 
   CRestoreMbrWindow optGui;
   DWORD dwDiskCount;
-  DWORD dwOriginalMbrNb;
-  DWORD dwCurrentMbrNb;
+  uintptr_t dwOriginalMbrNb;
+  uintptr_t dwCurrentMbrNb;
   char szHdList[2048];
   char szTemp[512];
   char szTemp2[512];
@@ -209,7 +211,7 @@ void restoreMbr(char *szImageFile, COptions *options)
 	  image.closeReading(true);
           throw excep;
 	}
-      SNPRINTF(szTemp, "%.3lu: %s [%llu blocks]", i, mbrOriginal[i].szDevice, mbrOriginal[i].qwBlocksCount);
+      SNPRINTF(szTemp, "%.3u: %s [%"PRIu64" blocks]", i, mbrOriginal[i].szDevice, mbrOriginal[i].qwBlocksCount);
       if (options->bBatchMode == false)
 	optGui.addMbr(szTemp, i);
       
@@ -321,6 +323,28 @@ void restoreMbr(char *szImageFile, COptions *options)
   RETURN;
 }
 
+struct portable_tm *localtime_portable (const time_t *__timer)
+{
+static struct portable_tm ret;
+struct tm *host_tm;
+
+	host_tm=localtime (__timer);
+	ret.tm_sec=host_tm->tm_sec;
+	ret.tm_min=host_tm->tm_min;
+	ret.tm_hour=host_tm->tm_hour;
+	ret.tm_mday=host_tm->tm_mday;
+	ret.tm_mon=host_tm->tm_mon;
+	ret.tm_year=host_tm->tm_year;
+	ret.tm_wday=host_tm->tm_wday;
+	ret.tm_yday=host_tm->tm_yday;
+	ret.tm_isdst=host_tm->tm_isdst;
+
+	ret.tm_gmtoff=host_tm->tm_gmtoff;
+	ret.tm_zone=0;
+
+	return &ret;
+}
+
 // =======================================================
 void savePartition(char *szDevice, char *szImageName, //, char *szFileSystem,
    COptions *options)
@@ -467,7 +491,7 @@ void savePartition(char *szDevice, char *szImageName, //, char *szFileSystem,
   time(&dt);
 
   // current time/date (date of image creation)
-  headMain.dateCreate = *localtime(&dt);
+  headMain.dateCreate = *localtime_portable(&dt);
 
   // file format version
   strcpy(headMain.szVersion, CURRENT_IMAGE_FORMAT); 
@@ -520,14 +544,14 @@ void savePartition(char *szDevice, char *szImageName, //, char *szFileSystem,
   if (strlen(options->szFullyBatchMode)>0)
   {
                  showDebug(3,"szFullyBatchMode=%s\n, len=%d",options->szFullyBatchMode,strlen(options->szFullyBatchMode));
-    fprintf(stderr,"partimage: status: Partimage: %s\n", headMain.szVersion);
+    fprintf(stderr,i18n("partimage: status: Partimage: %s\n"), headMain.szVersion);
     if (options->dwCompression == 0)
-      fprintf(stderr,"partimage: status: Image type: NONE\n");
-    else if (options->dwCompression == 1) +      fprintf(stderr,"partimage: status: Image type: GZIP\n");
+      fprintf(stderr,i18n("partimage: status: Image type: NONE\n"));
+    else if (options->dwCompression == 1) +      fprintf(stderr,i18n("partimage: status: Image type: GZIP\n"));
     else if (options->dwCompression == 2)
-      fprintf(stderr,"partimage: status: Image type: BZIP2\n");
+      fprintf(stderr,i18n("partimage: status: Image type: BZIP2\n"));
     else if (options->dwCompression == 3)
-      fprintf(stderr,"partimage: status: Image type: LZO\n");
+      fprintf(stderr,i18n("partimage: status: Image type: LZO\n"));
   }  
   try { image.write(&headMain, sizeof(CMainHeader), true); }
   catch ( CExceptions * excep )
@@ -923,15 +947,15 @@ void restorePartition(char *szDevice, char *szImageName, COptions *options)
   {
           showDebug(3,"szFullyBatchMode=%s\n, len=%d",options->szFullyBatchMode,strlen(options->szFullyBatchMode));
     options->dwCompression = (DWORD) image.getCompressionLevelForImage(szImageName);
-    fprintf(stderr,"partimage: status: Partimage: %s\n", headMain.szVersion);
+    fprintf(stderr,i18n("partimage: status: Partimage: %s\n"), headMain.szVersion);
     if (options->dwCompression == 0)
-      fprintf(stderr,"partimage: status: Image type: NONE\n");
+      fprintf(stderr,i18n("partimage: status: Image type: NONE\n"));
     else if (options->dwCompression == 1)
-      fprintf(stderr,"partimage: status: Image type: GZIP\n");
+      fprintf(stderr,i18n("partimage: status: Image type: GZIP\n"));
     else if (options->dwCompression == 2)
-      fprintf(stderr,"partimage: status: Image type: BZIP2\n");
+      fprintf(stderr,i18n("partimage: status: Image type: BZIP2\n"));
     else if (options->dwCompression == 3)
-      fprintf(stderr,"partimage: status: Image type: LZO\n");
+      fprintf(stderr,i18n("partimage: status: Image type: LZO\n"));
   }
   g_interface -> StatusLine(i18n("Restoring partition from the image file..."));
 
@@ -1121,7 +1145,7 @@ void closeFilesRestore(bool on_error, COptions options, CImage *image, FILE *fDe
 // =============================================================================
 bool isFileSystemSupported(char *szFileSystem)
 {
-  char *szPtr;
+  const char *szPtr;
   char szTemp[1024];
   WORD i;
 
@@ -1209,12 +1233,13 @@ void showDebugOptions(COptions *options)
 	    "options.bBatchMode = %d\n" 
 	    "options.bSplitWait = %d\n" 
 	    "options.bSync = %d\n" 
+	    "options.bMustLogin = %d\n" 
 	    "options.bUseSSL = %d\n" 
 	    "options.dwDebugLevel = %lu\n" 
 	    "options.bBackupMBR = %d\n" 
 	    "options.bSimulateMode = %d\n" 
 	    "options.szAutoMount = %s\n" 
-            "options.szFullyBatchMode = %s\n"
+        "options.szFullyBatchMode = %s\n"
 	    "============= end OPTIONS =====================\n\n\n",
 	    (int) options->bOverwrite, 
 	    //options->dwSplitMode,
@@ -1223,7 +1248,7 @@ void showDebugOptions(COptions *options)
 	    (int)options->bFailIfMounted, (int) options->bAskDesc, 
 	    options->dwFinish, options->szServerName, 
 	    options->dwServerPort, (int) options->bBatchMode, (int)options->bSplitWait,
-	    (int)options->bSync, (int)options->bUseSSL, options->dwDebugLevel, 
+	    (int)options->bSync, (int)options->bMustLogin, (int)options->bUseSSL, options->dwDebugLevel, 
             (int)options->bBackupMBR, (int)options->bSimulateMode, options->szAutoMount, options->szFullyBatchMode);
 }
 
@@ -1404,7 +1429,7 @@ int showUfsDisklabel(char *szDevice)
   FILE *fStream;
   char cBoot[512];
   CUFSDiskLabel dl;
-  int i=0, j;
+  int j;
   int nRes;
   CUFSPartition part;
   char letter='a';
@@ -1424,16 +1449,6 @@ int showUfsDisklabel(char *szDevice)
       RETURN_int(-1);
     }
 
-  // TODO: check the magic number
-  /*DWORD dwUfsMagic = UFS_MAGIC;
-  if (memcmp(cBuffer+5678, &dwUfsMagic, 4) != 0)
-    {
-      fclose(fStream);
-      RETURN_int(-1);
-    }*/
-
-  //for (i=0; (dl.d_magic == MAGIC) || (i==0); i++)
-    {
       nRes = fread(&dl, sizeof(CUFSDiskLabel), 1, fStream);
       if (nRes != 1)
 	  RETURN_int(-1);
@@ -1444,13 +1459,9 @@ int showUfsDisklabel(char *szDevice)
 	{
 	  SNPRINTF(szTemp, "%s%c", dl.d_typename, letter);
 	  part = dl.d_partitions[j];
-	  /*printf("    %s: sectors=%lu, startsect=%lu, fragsize=%lu, fragperblk=%lu, type=%s\n", szTemp, (DWORD)part.p_size, (DWORD)part.p_offset, (DWORD)part.p_fsize, (DWORD)part.p_frag, fstypenames[part.p_fstype]);*/
 	  letter++;
 	}
-	  
-      i++;
-    } 
-
+	    
   // close the partition
   fclose(fStream);
   return 0;
@@ -1461,13 +1472,13 @@ int detectFileSystem(const char *szDevice, char *szFileSys)
 {
   FILE *fDeviceFile;
   int nRes;
-  const int nBufSize = 128 * 1024; // 128 KB
+  const int nBufSize = 128 * 1024; // 128 KiB
   BYTE cBuffer[nBufSize];
 
   BEGIN;
 
   // init
-  strcpy(szFileSys, "-unknown-");
+  strcpy(szFileSys, i18n("-unknown-"));
 
   // 0. Open device
   fDeviceFile = fopen(szDevice, "rb");
@@ -1491,69 +1502,12 @@ int detectFileSystem(const char *szDevice, char *szFileSys)
   
   showDebug(4, "002 = after fread\n");
 
-  // **** check for UFS file system {magic @ 72020}
-  nRes = CUfsPartdetect(cBuffer, szFileSys, fDeviceFile);
+  //  **** check for XFS file system  [magic @ 0]
+  nRes = CXfsPartdetect(cBuffer, szFileSys);
   if (nRes > 0)
     return nRes;
 
-  /*DWORD dwUfsMagic = UFS_MAGIC;
-    if (memcmp(cBuffer+512, &dwUfsMagic, 4) == 0)
-    {
-      strcpy(szFileSys, "ufs");
-      return FS_UFS;
-    }
-  */
-  
-  fclose(fDeviceFile);
-
-  showDebug(4, "003 = after UFS\n");
-
-  // **** check for REISER3 file system [magic @ 65536]
-  nRes = CReiser3Partdetect(cBuffer, szFileSys);
-  if (nRes > 0)
-    return nRes;
-
- // **** check for REISER4 file system [magic @ 65536]
-  nRes = CReiser4Partdetect(cBuffer, szFileSys);
-  if (nRes > 0)
-    return nRes;
-
-  showDebug(4, "004 = after reiser\n");
-
-  //  **** check for JFS file system [magic @ 32768]
-  nRes = CJfsPartdetect(cBuffer, szFileSys);
-  if (nRes > 0)
-    return nRes;
-
-  showDebug(4, "005 = after Jfs\n");
-  
-  //  **** check for AFS file system [magic @ 1092]
-  nRes = CAfsPartdetect(cBuffer, szFileSys);
-  if (nRes > 0)
-    return nRes;
-
-  showDebug(4, "006 = after Afs\n");
-
-  //  **** check for EXT2FS file system [magic @ 1080]
-  nRes = CExt2Partdetect(cBuffer, szFileSys);
-  if (nRes > 0)
-    return nRes;
-
-  showDebug(4, "007 = after Ext2\n");
-
-  //  **** check for HFS file system [magic @ 1024]
-  nRes = CHfsPartdetect(cBuffer, szFileSys);
-  if (nRes > 0)
-    return nRes;
-
-  showDebug(4, "008 = after Hfs\n");
-
-  //  **** check for BeFS file system [magic @ 544]
-  nRes = CBefsPartdetect(cBuffer, szFileSys);
-  if (nRes > 0)
-    return nRes;
-
-  showDebug(4, "009 = after Befs\n");
+  showDebug(4, "012 = after Xfs\n");
 
   //  **** check for NTFS file system [magic @ 3]
   nRes = CNtfsPartdetect(cBuffer, szFileSys);
@@ -1569,18 +1523,66 @@ int detectFileSystem(const char *szDevice, char *szFileSys)
 
   showDebug(4, "011 = after Hpfs\n");
 
-  //  **** check for XFS file system  [magic @ 0]
-  nRes = CXfsPartdetect(cBuffer, szFileSys);
+  //  **** check for BeFS file system [magic @ 544]
+  nRes = CBefsPartdetect(cBuffer, szFileSys);
   if (nRes > 0)
     return nRes;
 
-  showDebug(4, "012 = after Xfs\n");
-  
+  showDebug(4, "009 = after Befs\n");
+
+  //  **** check for EXT2FS file system [magic @ 1080]
+  nRes = CExt2Partdetect(cBuffer, szFileSys);
+  if (nRes > 0)
+    return nRes;
+
+  showDebug(4, "007 = after Ext2\n");
+
+  //  **** check for HFS file system [magic @ 1024]
+  nRes = CHfsPartdetect(cBuffer, szFileSys);
+  if (nRes > 0)
+    return nRes;
+
+  showDebug(4, "008 = after Hfs\n");
+
+  //  **** check for AFS file system [magic @ 1092]
+  nRes = CAfsPartdetect(cBuffer, szFileSys);
+  if (nRes > 0)
+    return nRes;
+
+  showDebug(4, "006 = after Afs\n");
+
+  //  **** check for JFS file system [magic @ 32768]
+  nRes = CJfsPartdetect(cBuffer, szFileSys);
+  if (nRes > 0)
+    return nRes;
+
+  showDebug(4, "005 = after Jfs\n");
+
+  // **** check for REISER3 file system [magic @ 65536]
+  nRes = CReiser3Partdetect(cBuffer, szFileSys);
+  if (nRes > 0)
+    return nRes;
+
+ // **** check for REISER4 file system [magic @ 65536]
+  nRes = CReiser4Partdetect(cBuffer, szFileSys);
+  if (nRes > 0)
+    return nRes;
+
+  showDebug(4, "004 = after reiser\n");
+
+  // **** check for UFS file system {magic @ 72020}
+  nRes = CUfsPartdetect(cBuffer, szFileSys, fDeviceFile);
+  if (nRes > 0)
+    return nRes;
+
+  fclose(fDeviceFile);
+
+  showDebug(4, "003 = after UFS\n");
+
   // **** check for swap space
   char cTemp[512];
   memset(cTemp, 0, 511);
   memcpy(cTemp, cBuffer+getpagesize()-10, 10);
-  //*(cTemp+10) = '\0';
 
   if (strcmp(cTemp, "SWAP-SPACE")==0)
     {
@@ -1602,15 +1604,6 @@ int detectFileSystem(const char *szDevice, char *szFileSys)
       
   showDebug(4, "014 = after FAT\n");
 
-  // G. Extended ==> TODO
-  /*if (0)
-    {
-      fclose(fDeviceFile);
-      strcpy(szFileSys, "extended");
-      delete []cBuffer;
-      return FS_EXTENDED;
-    }*/
-    
   // end on error
   RETURN_int(-1);
 }
@@ -1678,7 +1671,7 @@ int checkInodeForDevice(char *szDevice, char *szFullyBatchMode) // NEW FUNCTION
       if(strlen(szFullyBatchMode)<=0)
       {
           nRes = g_interface->msgBoxYesNo(i18n("inode missing"), i18n("%s inode doesn't exists. Partimage can create it for you. You can also use the manual mknod "
-								  "command. Do you want this inode to be created for you now ?"), szDevice);
+								  "command. Do you want this inode to be created for you now?"), szDevice);
           if (nRes == MSGBOX_NO)
 	      RETURN_int(-1);
       }
@@ -1743,6 +1736,7 @@ int isDevfsMounted()
 {
 	FILE *f;
 	char buf[256];
+	int nRes __attribute__ ((unused));
 
 	if (access("/proc/mounts",R_OK)==-1)
 	{
@@ -1758,7 +1752,7 @@ int isDevfsMounted()
 
 	while (!feof(f))
 	{
-		fscanf(f,"%255s",buf); *(buf+255)='\0';
+		nRes = fscanf(f,"%255s",buf); *(buf+255)='\0';
 		if (!strcmp(buf,"devfs"))
 		{
 			fclose(f);
@@ -1865,7 +1859,7 @@ char *decodePartitionEntry(char *cPtr, int *nMajor, int *nMinor,
     cPtr++;
   memset(szDevice, 0, 127);
   cCur = szDevice;
-  while(isalnum(*cPtr) || *cPtr == '/')
+  while(isgraph(*cPtr))
     {
       *cCur = *cPtr;
       cCur++;
@@ -1946,7 +1940,7 @@ int checkOptions(COptions options, char *szDevice, char *szImageFile)
   //if ((options.dwSplitMode == SPLIT_SIZE) && (options.qwSplitSize < (QWORD)(1024*1024)))
   if ((options.qwSplitSize) && (options.qwSplitSize < (QWORD)(1024*1024)))
     {
-      g_interface->msgBoxError(i18n("Split volumes size must be an integer greater than 1024 (size >= 1024 KB)"));	
+      g_interface->msgBoxError(i18n("Split volumes size must be an integer greater than 1024 (size >= 1024 KiB)"));	
       RETURN_int(-1);
     }
 
@@ -1996,6 +1990,13 @@ int CReiser3Partdetect(BYTE *cBootSect, char *szFileSystem)
 {
   BEGIN;
   
+/*  static int dv=0;
+char bn[1024];
+sprintf(bn,"BOOTIN_%d",dv); dv++;
+FILE* fpp=fopen(bn,"wb");
+fwrite(cBootSect,128000,1,fpp);
+fclose(fpp);
+*/
   if (memcmp(cBootSect+REISERFS_DISK_OFFSET_IN_BYTES+52, REISERFS_SUPER_MAGIC_STRING, strlen(REISERFS_SUPER_MAGIC_STRING)) == 0)	
     {
       strcpy(szFileSystem, "reiserfs-3.5");
@@ -2009,7 +2010,7 @@ int CReiser3Partdetect(BYTE *cBootSect, char *szFileSystem)
       strcpy(szFileSystem, "reiserfs-3.6");
       return FS_REISER_3_6;
     }
-  
+
   showDebug(4, "002 = after check for Reiser-3.6\n");
 
   RETURN_int(0); // Not a REISER partition
@@ -2243,6 +2244,8 @@ int CUfsPartdetect(BYTE *cBootSect, char *szFileSystem, FILE *fDeviceFile)
   cylinderGroupHeader cgp;
   DWORD i;
   int nRes;
+  off_t off;
+  u_int dev_bsize = 512; /* XXX check block size in the disk label */
 
   // init
   memcpy(&sbUfs, cBootSect+8192, sizeof(sbUfs));
@@ -2252,7 +2255,9 @@ int CUfsPartdetect(BYTE *cBootSect, char *szFileSystem, FILE *fDeviceFile)
 
   for (i=0; i < (DWORD)sbUfs.fs_ncg; i++)
     {
-      nRes = bigFseek(fDeviceFile, ufs_cgcmin_sb(sbUfs,i)*1024);
+      //nRes = bigFseek(fDeviceFile, ufs_cgcmin_sb(sbUfs,i)*1024);
+      off = (off_t)fsbtodb(&sbUfs, cgtod(&sbUfs, i)) * dev_bsize;
+      nRes = bigFseek(fDeviceFile, off);
       if (nRes == -1)
 	{
 	  showDebug(2, "DETECT_UFS: bigFseek() failed\n");
@@ -2348,7 +2353,7 @@ static char *sfdisk_line_to_disk_device(const char *line)
 
 static char *sfdisk_line_to_partition_device(const char *line)
 {
-  if (char *p = strchr(line, ':')) {
+  if (const char *p = strchr(line, ':')) {
     while (p[-1] == ' ' && p > line) p--;
     return strndup(line, p - line);
   } else {
@@ -2368,7 +2373,7 @@ static char *full_device_to_device(char *device)
   }
   p = strdup(p);
   char *q;
-  while (q = strchr(p, '/')) *q = '_';
+  while ((q = strchr(p, '/'))!=NULL) *q = '_';
 
   return p;
 }
@@ -2480,6 +2485,7 @@ void rest_all(char *szImageDir, COptions *options)
 
   char **sfdisk;
   unsigned int sfdisk_nb_lines = 0;
+  int nRes __attribute__ ((unused));
 
   char aux[MAXPATHLEN];
   CImage image(options);
@@ -2550,10 +2556,60 @@ void rest_all(char *szImageDir, COptions *options)
     } else if (strstr(sfdisk[i], "Id=82")) {
       SNPRINTF(aux, "mkswap %s >>" PARTIMAGE_LOG, device);
       showDebug(2, "%s\n", aux);
-      system(aux);
+      nRes = system(aux);
     }
     free(device);
   }
   
   RETURN;
 }
+
+void save_file(char *szImageFile, COptions *options)
+{
+  BEGIN;  
+  int in_offset = 0;
+  char *in = (char *) malloc(MAXPATHLEN);
+
+  CImage image(options);
+  image.set_szImageFilename(szImageFile);
+  image.openWriting();
+
+  while (1) {
+    int len = fread(in + in_offset, 1, MAXPATHLEN, stdin);
+    showDebug(1, "read %d chars on stdin\n", len);
+    if (!len) break;
+    in = (char *) realloc(in, MAXPATHLEN + (in_offset += len));
+  }
+  showDebug(1, "writing %d chars in %s\n", in_offset, szImageFile);
+  write_unsigned(&image, in_offset);
+  image.write((void *) in, in_offset, true);
+  free(in);
+
+  closeFilesSave(false, *options, &image, NULL);
+
+  RETURN;
+}
+
+void rest_file(char *szImageFile, COptions *options)
+{
+  BEGIN;  
+  size_t stRet __attribute__ ((unused)) ;
+
+  CImage image(options);
+
+  {
+    image.set_szImageFilename(szImageFile);
+    image.openReading();
+
+    unsigned int len = read_unsigned(&image);
+    char *s = (char *) malloc(len + 1);
+    image.read(s, len, true);
+    stRet = fwrite(s, 1, len, stdout);
+    free(s);
+    
+    // since sfdisk.lst is not really nice to the library, we can't close the thread from procReadBufferFromImage which has already exited
+    image.closeReading(false);
+  }
+  
+  RETURN;
+}
