@@ -21,7 +21,10 @@
 #include <errno.h>
 #include <libintl.h> // intl translation
 #include <stdio.h>
+#include <stdint.h>
 #include <time.h>
+#define __STDC_FORMAT_MACROS
+#include <inttypes.h>
 
 #ifdef HAVE_CONFIG_H
   #include <config.h>
@@ -44,7 +47,7 @@
 #define THROW(nError, args...) showDebug(1, "THROW: %d\n", nError),throw new CExceptions(__FUNCTION__, nError, ## args)
 #define SNPRINTF(szDest, szFormat, args...) snprintf(szDest, sizeof(szDest)-1, szFormat, ## args),szDest[sizeof(szDest)-1]=0 // allow to use this macro in 'else'
 
-#define COPYBUF_SIZE 262144 // 256 KB
+#define COPYBUF_SIZE 262144 // 256 KiB
 #define MAX_FREE_BLOCKS_SKIPPED 2048
 
 // time
@@ -53,10 +56,10 @@ extern time_t g_timeStart; // Beginning of the copy, after confirmation
 extern time_t g_timeEnd;   // End of the operation
 
 typedef bool BOOL; // variant size
-typedef unsigned char BYTE; // 8 bits
-typedef unsigned short int WORD; // 16 bits
-typedef unsigned long int DWORD; // 32 bits
-typedef unsigned long long int QWORD; // 64 bits
+typedef uint8_t BYTE; // 8 bits
+typedef uint16_t WORD; // 16 bits
+typedef uint32_t DWORD; // 32 bits
+typedef uint64_t QWORD; // 64 bits
 
 #include "endianess.h"
 
@@ -96,6 +99,8 @@ extern bool g_bSigKill;
 #define OPERATION_EXIT       5
 #define OPERATION_SAVE_ALL   6
 #define OPERATION_REST_ALL   7
+#define OPERATION_SAVE_FILE  8
+#define OPERATION_REST_FILE  9
 
 // returned by CExceptionsGUI::windowError 
 // user canceled the job
@@ -189,6 +194,7 @@ extern bool g_bSigKill;
 #define OPTIONS_DEFAULT_SPLIT_WAIT            false
 #define OPTIONS_DEFAULT_RUN_SHELL             false
 #define OPTIONS_DEFAULT_SYNC                  true
+#define OPTIONS_DEFAULT_LOGIN                 true
 #define OPTIONS_DEFAULT_SSL                   true
 #define OPTIONS_DEFAULT_BACKUP_MBR            true
 #define OPTIONS_DEFAULT_SIMULATE_MODE         false
