@@ -51,7 +51,7 @@
 #include "buffer.h"
 #include "pathnames.h"
 
-#define SPACEFILE_SIZE (10*1024*1024) // 10 MB
+#define SPACEFILE_SIZE (10*1024*1024) // 10 MiB
 
 #ifndef HAVE_FSTAT64
   #warning "fstat32 used"
@@ -208,7 +208,7 @@ void CImageDisk::write(void *vBuf, DWORD dwLength, int *nResult) // [Buffer-Thre
 	      showDebug(3, "ERROR: ERR_INCOMPLETEFINISH (dwRes != dwLength-dwFirstWritten) error %d = %s "
 			"[dwRes=%lu and dwLength-dwFirstWritten=%lu]\n", 
 			errno, strerror(errno), dwRes, dwLength-dwFirstWritten);
-	      newtWinMessage("error", "ok", "ERROR: ERR_INCOMPLETEFINISH (dwRes != dwLength-dwFirstWritten) "
+	      newtWinMessage((char *)"error", (char *)"ok", (char *)"ERROR: ERR_INCOMPLETEFINISH (dwRes != dwLength-dwFirstWritten) "
 			     "error %d = %s [dwRes=%lu and dwLength-dwFirstWritten=%lu]\n", 
 			     errno, strerror(errno), dwRes, dwLength-dwFirstWritten);
 	    }
@@ -306,7 +306,7 @@ int CImageDisk::getCompressionLevelForImage(char *szFilename) // [Main-Thread]
   BZFILE *bzImageFile;
   DWORD dwRes;
   CVolumeHeader headVolume;
-  char *szLabel = MAGIC_BEGIN_VOLUME;
+  const char *szLabel = MAGIC_BEGIN_VOLUME;
   char cBuf[64];
   
   showDebug(3, "TRACE_001\n");
@@ -524,7 +524,7 @@ void CImageDisk::openWriting() // [Main-Thread]
   if (m_bIsOpened == true)
     THROW(ERR_OPENED);
   
-  SNPRINTF(szShortName, "pi%.8lx", (DWORD)generateIdentificator());
+  SNPRINTF(szShortName, "pi%.8x", (DWORD)generateIdentificator());
   extractFilepathFromFullPath(m_szImageFilename, szPathDir); // filepath without filename
   if (access(szPathDir, R_OK) == -1) mkdir(szPathDir, 0755);
   snprintf(m_szSpaceFilename, MAXPATHLEN, "%s/%8s.tmp", szPathDir, szShortName);
@@ -572,11 +572,12 @@ void CImageDisk::openWriting() // [Main-Thread]
       m_nImageFd = open(m_szImageFilename, flags, S_IREAD | S_IWRITE);
     #endif
       if ((m_nImageFd == -1))
+	  {
         if (errno == EEXIST)
           THROW(ERR_EXIST);
         else
           THROW(ERR_ERRNO, errno);
-
+	  }
       // ---- avoid RACE CONDITIONS security problems:
     #ifdef HAVE_FSTAT64
       if (fstat64(m_nImageFd, &fStat) == -1)
@@ -764,7 +765,7 @@ void CImageDisk::getDiskFreeSpaceForFile(QWORD *qwAvailDiskSpace, char *szFilepa
 #else // HAVE_SYS_STATFS_H
 void CImageDisk::getDiskFreeSpaceForFile(QWORD *qwAvailDiskSpace, char *szFilepath) // [Main-Thread]
 {
-  #warning "getDiskFreeSpaceForFile always return 5 MB"
+  #warning "getDiskFreeSpaceForFile always return 5 MiB"
   *qwAvailDiskSpace = (QWORD)(5LL*1024LL*1024LL);
   return 0;
 }
